/*
 * Furo_3Liion_mega8L.asm
 *
 *  Created: 2016.04.21. 19:49:11
 *   Author: admin
 */ 
.include "m8def.inc"

;Debug mode switches. Value usually is not important. Exists or doesn't exist. e.g. DEFINED(debugmode) = true or fasle value is important! Use ";" to disable debug mode.
;.equ debugmode = 123 ; general debug switch
;.equ debugdelay = 654; delay quits fast
;.equ debugadc = 234; using predefined test results. Use DEBUG_ADC_RESULT
;;;.equ debugdrain = -34 ; FET Drain state emulation
;.equ debugusart = 1 ; send data via usart. Value: 1 - write caracter to sram virtual screen for simulator.
;.equ debugdiag = 0 ; terminal mode. Value: 1 - Forced diagnostic mode for simulator. 

.if defined(debugmode)
	.warning "."
	.warning "DEBUG MODE"
	.message "msg DEBUG MODE"
	.if defined(debugdelay)
		.warning " - DEBUG DELAY ON"
	.endif
	.if defined(debugadc)
		.warning " - DEBUG ADC ON"
	.endif
	.if defined(debugdrain)
		.warning " - DEBUG DRAIN ON"
	.endif
	.if defined(debugusart)
		.warning " - DEBUG USART ON"
	.endif
	.if defined(debugdiag)
		.warning " - DEBUG DIAG ON"
		.if debugdiag == 1
			.warning "-  FORCED DIAGNOSTIC MODE ON (debugdiag==1). Only for simulator. Don't use on real device!"
		.endif
	.endif
	.warning "."
.else
	.warning "NORMAL MODE"
.endif


; Delay parameters
.equ SYSCLK=1000000 ; system clock
.equ MILLISEC=int(SYSCLK/(1+2)*0.001) ; determine 1 ms for sub delay
.def DELAY_1=r16 ; registers for sub delay
.def DELAY_2=r17
.def DELAY_3=r18
;.def DELAY_4=r19

.equ VBG_MAX=int(1023*1.4/5)+10 ; Max bandgap voltage ADC result
.equ VGND_MAX=10; MAX GND offset

.equ R_HEATSINK_REF=1500 ; Reference resistor value for heatsink temperature
.equ R_HEATSINK_MIN=802 ; Resistance of PTC on 0 C (802 ohm - 828 ohm)
.equ R_HEATSINK_MAX=1416 ; Resistance of PTC on 70 C (1369 ohm - 1416 ohm)
.equ V_HEATSINK_MIN=int(1023*(R_HEATSINK_MIN/(1.0*R_HEATSINK_MIN+R_HEATSINK_REF))) ; lowest good heatsink value
.equ V_HEATSINK_MAX=int(1023*(R_HEATSINK_MAX/(1.0*R_HEATSINK_MAX+R_HEATSINK_REF))) ; highest good heatsink value

.equ R_BT_REF=1500 ; Reference resistor value for battery temperature
.equ R_BT_MIN=802  ; Resistance of PTCs on 0 C (802 ohm - 828 ohm)
.equ R_BT_MAX=1278 ; Resistance of PTCs on 60 C worst case (1278 ohm - 1319 ohm)
.equ BT_RATIO_MIN=int(256.0*R_BT_MIN/R_BT_REF) ; Multiple factor for counting minimum value. Utx_min=Utref*BT_RATIO_MIN/256
.equ BT_RATIO_MAX=int(256.0*R_BT_MAX/R_BT_REF) ; Multiple factor for counting maximum value. Utx_max=Utref*BT_RATIO_MAX/256

.equ V_BATTERY_MIN = int(2.5*4.7/(4.7+10.2)*1023/5); Umin=2.5V, Uref=5V, R1=4.7K, R2=10,2K

.equ V_BATTERIES_MAX = int(12.6*4.7/(4.7+10.2)*1023/5); Umax=3*4.2=12.6V, Uref=5V, R1=4.7K, R2=10,2K
.equ V_BATTERIES_NOMINAL = int(10.8*4.7/(4.7+10.2)*1023/5); Unominal=3*3.6=10.8V, Uref=5V, R1=4.7K, R2=10,2K
.equ V_BATTERIES_LOW = int(9*4.7/(4.7+10.2)*1023/5); Ulow=3*3.0=9V, Uref=5V, R1=4.7K, R2=10,2K
.equ V_DRILL_NOISE = int(0.031*4.7/(4.7+10.2)*1023/5); Unoise=0.031V, Uref=5V, R1=4.7K, R2=10,2K -> 2
.equ V_DROP = int(0.25*4.7/(4.7+10.2)*1023/5); Udrop=0.25V, Uref=5V, R1=4.7K, R2=10,2K

; control bits
.equ DIAG_MODE_OUT = PD6 ;	Open PD6 and PD7: normal mode
.equ DIAG_MODE_IN = PD7 ;	Close PD6 and PD7 with jumper: diagnostic mode

.equ PWR_SW = PB0 ; POWER switch. 0 - Enable, 1 - Disable
.equ GATE_SW = PD2 ; Gate switch. 0 - Enable, 1 - Disable
.equ BUZZER_SW = PD4; Buzzer switch. 0 - Enable, 1 - Disable
; cd4051 control bits
.equ CMUX_A = 1<<PB5
.equ CMUX_B = 1<<PB4
.equ CMUX_C = 1<<PB3
.equ CMUX_INH = PB2 ; 0 - Enable, 1 - Disable MUX

; cd4051 channel selection
.equ CM_BT_H = CMUX_B ; X2 Upper battery's Voldage
.equ CM_BT_M = CMUX_A ; X1 Midle battery's Voltage
.equ CM_BT_L = 0 ; X0 Lower battery's Voltage
.equ CM_BT_0 = CMUX_B | CMUX_A ; X3 0V Voltage
.equ CM_DRAIN_D = CMUX_C | CMUX_B; X6 Drain Voltage
.equ CM_DRAIN_T = CMUX_C ; X4 Drain controlled transistor Voltage
.equ CM_EMPTY1 = CMUX_C | CMUX_A ; X5 Not connected
.equ CM_EMPTY2 = CMUX_C | CMUX_B | CMUX_A ; X7 Not connected
.equ CM_MASK = ~(CMUX_C | CMUX_B | CMUX_A) ; Mask for amux

; ADMUX channel selection
.equ AM_HEATSINK = 0 ; Heatsink PTC, ADC0
.equ AM_CMUX = (1<<MUX0) ; CD4051 out, ADC1
.equ AM_BT1 = (1<<MUX1) ; Battery PTC1, ADC2
.equ AM_BT2 = (1<<MUX1) | (1<<MUX0) ; Battery PTC2, ADC3
.equ AM_BT3 = (1<<MUX2) ; Battery PTC3, ADC4
.equ AM_ADC5 = (1<<MUX2) | (1<<MUX0) ; ADC5 
.equ AM_ADC6 = (1<<MUX2) | (1<<MUX1) ; ADC6
.equ AM_ADC7 = (1<<MUX2) | (1<<MUX1) | (1<<MUX0); ADC7
.equ AM_VBG = (1<<MUX3) | (1<<MUX2) | (1<<MUX1) ; Bandgap Voltage ~1.3V
.equ AM_GND = (1<<MUX3) | (1<<MUX2) | (1<<MUX1) | (1<<MUX0) ; GND
.equ AM_MASK =~((1<<MUX3) | (1<<MUX2) | (1<<MUX1) | (1<<MUX0)) ; MUXx mask

; Error codes
.def ERROR_REG = r31
.def ROUND_REG = r30
.equ ERROR_OK = 0

.equ ERROR_BTU_VOLTAGE_LOW = 1
.equ ERROR_BTM_VOLTAGE_LOW = 2
.equ ERROR_BTL_VOLTAGE_LOW = 3
.equ ERROR_BANDGAP = 4

.equ ERROR_HEATSINK_HIGH = 5
.equ ERROR_BT1_HEAT_HIGH = 6
.equ ERROR_BT2_HEAT_HIGH = 7
.equ ERROR_BT3_HEAT_HIGH = 8

.equ ERROR_HEATSINK_LOW = 9
.equ ERROR_BT1_HEAT_LOW = 10
.equ ERROR_BT2_HEAT_LOW = 11
.equ ERROR_BT3_HEAT_LOW = 12

.equ ERROR_BTU_VOLTAGE_NEGATIVE = 13
.equ ERROR_BTM_VOLTAGE_NEGATIVE = 14
.equ ERROR_BTL_VOLTAGE_NEGATIVE = 15


;Dianostic mode constans:
.equ DIAG_FET = 0;
.equ DIAG_INH = 1;
.equ DIAG_BUZZER = 2;
.equ DIAG_ECHO = 3;
.equ DIAG_MEASURE_MODE = 4
.equ DIAG_FIRST_RUN = 5

.equ DIAG_FET_S = (1<<DIAG_FET)
.equ DIAG_INH_S = (1<<DIAG_INH)
.equ DIAG_BUZZER_S = (1<<DIAG_BUZZER)
.equ DIAG_ECHO_S = (1<<DIAG_ECHO)
.equ DIAG_MEASURE_MODE_S = (1<<DIAG_MEASURE_MODE)
.equ DIAG_FIRST_RUN_S = (1<<DIAG_FIRST_RUN)
;General purpose macros

;----------------------------------
;CP16: compare two words in registers
; Usage: CP16 @0,@1,@2,@3 -> CP @1:@0,@3:@2
; @0-@4 : registers
;
.MACRO CP16
	cp @1,@3
	brne cp16_e
	cp @0,@2
	cp16_e:
.ENDMACRO

;------------------------------------
;CPI16: compare word in register with immediate
; Usage: CPI16 @0,@1,@2 -> cpi r@1:r@0,i@2
; @0,@1:	registers
; @2:		immediate
;	
.MACRO CPI16 ; 
	cpi @1,HIGH(@2)
	brne cpi16_e
	cpi @0,LOW(@2)
	cpi16_e:
.ENDMACRO

;------------------------------------------
;DELAY_MILLISEC: perform a delay of @0 milliseconds
; Usage: DELAY_MILLISEC @0
; @0: Immediate delay time in millisec
;
.MACRO DELAY_MILLISEC
	.set MILLI_CNT=MILLISEC*@0
	ldi DELAY_1,LOW(MILLI_CNT)
	ldi DELAY_2,BYTE2(MILLI_CNT)
	ldi DELAY_3,BYTE3(MILLI_CNT)
	;ldi DELAY_4,BYTE4(MILLI_CNT)
	rcall delay
.ENDMACRO

;---------------------------------------------------
;STS_WORD: move registers (@2:@1) to sram (@0)
; Usage: STS_WORD @0,@1,@2
; @0: destination sram address
; @0,@1: source registers
;
.MACRO STS_WORD 
	sts @0,@1
	sts @0+1,@2
.ENDMACRO

;---------------------------------------------------
;LDS_WORD: move sram contents (@2) to registers (@1:@0) 
; Usage: LDS_WORD @0,@1,@2
; @0,@1: destination registers
; @2: source sram address
;
.MACRO LDS_WORD 
	lds @0,@2
	lds @1,@2+1
.ENDMACRO

;----------------------------------------------------------------
;ROUND_INIT: Clear round counter
; Usage: ROUND_INIT
;
.MACRO ROUND_INIT
	clr ROUND_REG
	sts sram_round,ROUND_REG
.ENDMACRO

;---------------------------------------------------------------
;ROUND_CHK: Repeat checking routine if it reports failure until round counter reaches the limit
; Usage: ROUND_CHK @0,@1,@2
; Parameters:
;	@0 - immediate: Maximum number of repeating
;	@1 - .cseg label: jump here for repeat checking
;	@2 - .cseg label: jump here after last round
;
.MACRO ROUND_CHK
	cpi r31,ERROR_OK; check error status
	breq round_chk_end
	lds ROUND_REG,sram_round
	inc ROUND_REG
	sts sram_round,ROUND_REG
	cpi ROUND_REG,@0
	brsh round_chk_err
	rjmp @1; try again
	round_chk_err:
	rjmp @2; jump to error handling
	round_chk_end:
.ENDMACRO

; Hardware specified macros

;----------------------------------------
;SET_CMUX: set the input of cd4051 analog multiplexer and enable it
; Usage: SET_CMUX @0
; @0: constant, address of selected channel
;
.MACRO SET_CMUX
	ldi r16,@0
	rcall sel_cmux
.ENDMACRO

;----------------------------------------
;SET_AMUX: select the ADC channel
; Usage: SET_AMUX @0
; @0: constant, address of selected channel
;
.MACRO SET_AMUX
	ldi r16,@0
	rcall sel_amux
.ENDMACRO

;------------------------------------------
;ADC_READ: get DC result from adc
; Usage: ADC_READ @0,@1
;	@0: destination register pair
;	@1: destination sram address
;
.MACRO ADC_READ
	rcall read_adc
	movw @0,r16 ; copy two register wide result
	STS_WORD @1,r16,r17 ; DC component
.ENDMACRO

;-------------------------------------------
;ADC_READ_ALL: get DC, AC , peak to peak results from adc
; Usage: ADC_READ_ALL @0,@1,@2,@3
;	@0: destination register pair for DC component
;	@1: destination sram address for DC component
;	@2:	destination sram address for AC component
;	@3:	destination sram address for peak to peak value
;
.MACRO ADC_READ_ALL
	ADC_READ @0,@1
	STS_WORD @2,r18,r19 ; AC component
	sub r20,r22
	sbc r21,r23
	STS_WORD @3,r20,r21 ; Peak to peak
.ENDMACRO

;------------------------------------------------
;USART_SEND_CRAR: send one character to the terminal
; Usage: USART_SEND_CHAR @0
;	@0: immediate
;
.MACRO USART_SEND_CHAR
	push r16
	ldi r16,@0
	rcall usart_send_byte
	pop r16
.ENDMACRO

;------------------------------------------------
;USART_SEND_TXT: sent zero terminated text to the terminal
; Usage: USART_SEND_TEXT @0
; @0: .cseg address of text
;
.MACRO USART_SEND_TXT
	ldi r26,low(@0)
	ldi r27,high(@0)
	rcall usart_send_text
.ENDMACRO

;-----------------------------------------------------------------
;Usage: DEBUG_SEND_TXT @0
.MACRO DEBUG_SEND_TXT ;send text from .cseg when debugmode on
	.if defined(debugmode) && defined(debugusart)
		USART_SEND_TXT @0
	.endif
.ENDMACRO

;-------------------------------------------------
;USART_SEND_RAM: send 64 word size value from sram
; Usage: USART_SEND_RAM: @0,@1
;	@0: .cseg address of header text
;	@1:	.dseg start address of sram area
.MACRO USART_SEND_SRAM ;@0 text, @1 sram location
	USART_SEND_TXT @0
	USART_SEND_TXT txt_crlf
	USART_SEND_TXT txt_samples
	ldi r26,low(@1)
	ldi r27,high(@1)
	rcall usart_send_sram_fn
	USART_SEND_TXT txt_crlf2
.ENDMACRO

;-----------------------------------------------------------------
;Usage: DEBUG_SEND_SRAM @0,@1
.MACRO DEBUG_SEND_SRAM ;@0 text, @1 sram location
	.if defined(debugmode) && defined(debugusart)
		USART_SEND_SRAM @0,@1
	.endif
.ENDMACRO

;-------------------------------------------------
;USART_SEND_BIN_REG: send the content of a register in binary format
; Usage: USART_SEND_BIN_REG @0
;	@0: source register
;
.MACRO USART_SEND_BIN_REG
	USART_SEND_TXT txt_bin
	push r16
	mov r16,@0
	rcall usart_send_bin_byte
	pop r16
.ENDMACRO

;-------------------------------------------------
;USART_SEND_BIN_I8: send 8bit wide immediate in binary format
; Usage: USART_SEND_BIN_I8 @0
;	@0: immediate
;
.MACRO USART_SEND_BIN_I8
	USART_SEND_TXT txt_bin
	push r16
	ldi r16,@0
	rcall usart_send_bin_byte
	pop r16
.ENDMACRO

;----------------------------------------------------
;USART_SEND_REGISTER: send register value in hex format
; Usage USART_SEND_REGISTER @0
;	@0: source register
;
.MACRO USART_SEND_REGISTER
	USART_SEND_TXT txt_hex
	push r16
	mov r16,@0
	rcall usart_send_hex_byte
	pop r16
.ENDMACRO

;----------------------------------------------------------------
;Usage: DEBUG_SEND_REGISTER @0
.MACRO DEBUG_SEND_REGISTER; Send register value to usart in debug mode
	.if defined(debugmode) && defined(debugusart)
		USART_SEND_REGISTER @0
	.endif
.ENDMACRO

;-------------------------------------------------------
;USART_SEND_REGISTERS: send word in hex format from registers
; Usage: USART_SEND_REGISTERS @0,@1
;	@0: register, low byte
;	@1: register, high byte
;
.MACRO USART_SEND_REGISTERS 
	USART_SEND_TXT txt_hex
	push r16
	push r17
	mov r16,@0
	mov r17,@1
	rcall usart_send_hex_word
	pop r17
	pop r16
.ENDMACRO

;----------------------------------------------------------------
;Usage: DEBUG_SEND_REGISTERS @0,@1
.MACRO DEBUG_SEND_REGISTERS; Send register pair value to usart in debug mode
	.if defined(debugmode) && defined(debugusart)
		USART_SEND_REGISTERS @0,@1
	.endif
.ENDMACRO

;------------------------------------------------------
;USART_SEND_IMMEDIATE: send 16bit immediate in hex format
; Usage: USART_SEND_IMMEDIATE @0
;	@0: immediate
;
.MACRO USART_SEND_IMMEDIATE 
	USART_SEND_TXT txt_hex
	push r16
	push r17
	ldi r16,low(@0)
	ldi r17,high(@0)
	rcall usart_send_hex_word
	pop r17
	pop r16
.ENDMACRO

;--------------------------------------------------------------
;Usage: DEBUG_SEND_IMMEDIATE @0
.MACRO DEBUG_SEND_IMMEDIATE; Send immediate to usart in debug mode
	.if defined(debugmode) && defined(debugusart)
		USART_SEND_IMMEDIATE @0
	.endif
.ENDMACRO

;--------------------------------------------------------
;USART_SEND_ERROR: send error message when status == error code
; Usage: USART_SEND_ERROR @0,@1
;	@0: immediate, error code
;	@1:	address of error message
;
.MACRO USART_SEND_ERROR
	cpi r31,@0
	brne u_s_err10
	USART_SEND_TXT @1
	u_s_err10:
.ENDMACRO

;-------------------------------------------------------
;USART_SEND_VALUE_E0: send 16bit wide value from sram in hex format
; Usage: USART_SEND_VALUE_E0 @0
;	@0: .dseg address
;
.MACRO USART_SEND_VALUE_E0
	LDS_WORD r16,r17,@0
	USART_SEND_REGISTERS r16,r17
.ENDMACRO

;---------------------------------------------------------
;USART_SEND_VALUE_E1: Send one error message and one value from sram
; Usage: USART_SEND_VALUE_E1 @0,@1,@2
;	@0: .dseg address
;	@1: error code
;	@2: error message .cseg address
;
.MACRO USART_SEND_VALUE_E1
	USART_SEND_ERROR @1,@2
	USART_SEND_VALUE_E0 @0
.ENDMACRO

;-----------------------------------------------------------
;USART_SEND_VALUE_E2: Send one of two error message and one value from sram
; Usage: USART_SEND_VALUE_E2 @0,@1,@2,@3,@4
;	@0: .dseg address
;	@1: error code1
;	@2: error message 1 .cseg address
;	@3: error code2
;	@4: error message 2 .cseg address
;
.MACRO USART_SEND_VALUE_E2
	USART_SEND_ERROR @3,@4
	USART_SEND_VALUE_E1 @0,@1,@2
.ENDMACRO

;----------------------------------------------------------------------
;Usage: DEBUG_DIAG
.MACRO DEBUG_DIAG ; SKIP next instruction in debug mode if diagmode on
	.if defined(debugmode) && defined(debugdiag)
		push r16
		rcall diagmode_chk
		pop r16
		breq debug_diag10 ; diagmode off
		cpse r16,r16
		debug_diag10:
	.endif
.ENDMACRO

;--------------------------------------------------------------------------------
;Usage: DEBUG_ADC_RESULT @0,@1
.MACRO DEBUG_ADC_RESULT ; ADC gets @0 and @1 instead of real results in debug mode
	.if defined(debugmode) && defined(debugadc)
		push r16
		ldi r16,low(@0)
		sts sram_adc_debug_1,r16
		ldi r16,high(@0)
		sts sram_adc_debug_1+1,r16
		ldi r16,low(@1)
		sts sram_adc_debug_2,r16
		ldi r16,high(@1)
		sts sram_adc_debug_2+1,r16
		pop r16
	.endif
.ENDMACRO

;-------------------------------------------------------------------
;Usage: DEBUG_ADC_RESULT_DC @0
.MACRO DEBUG_ADC_RESULT_DC ;ADC get only DC component in debug mode
	DEBUG_ADC_RESULT @0,@0
.ENDMACRO

;-------------------------------------------------------------------
;Do not use it!
.MACRO DELETED_DEBUG_ADC_USART ;Store ADC results in SRAM from @0 
	.if defined(debugmode) && defined(debugusart)
		ldi r26,low(@0)
		ldi r27,high(@0)
	.endif
.ENDMACRO

;------------------------------------------------------------------------------
;Battery temperature checking
; Parameters: 
;	in registers (@0+1):@0:	sensor upper pin ADC value
;	in registers @2:@1:		sensor lower pin ADC value
;	immediate 8bit @3:		"too low" error code
;	immediate 8bit @4:		"too high" error code
;	out .dseg label @5:		address of calculated sensor value
;	.cseg label @6:			jump here if out of range 
;
.MACRO CHK_BT_HEAT
	movw r16,@0 ; BT value
	sub r16,@1
	sbc r17,@2
	STS_WORD @5,r16,r17
	
	CP16 r16,r17,r14,r15 ; CP BT_HEAT,BT_MIN
	brsh chk_bt_heat_20
	ldi ERROR_REG,@3
	DEBUG_DIAG
	rjmp @6 ; Error: Too low	

	chk_bt_heat_20:
	CP16 r12,r13,r16,r17 ; CP BT_MAX,BT_HEAT
	brsh chk_bt_heat_30
	ldi ERROR_REG,@4
	DEBUG_DIAG
	rjmp @6 ; Error: too high
	chk_bt_heat_30: ; in allowed range
.ENDMACRO

;-------------------------------------------------------------
;Battery voltage checking
; Parameters: 
;	in registers (@0+1):@0:	Battery positive pin value
;	in registers @2:@1:		Battery negative pin value
;	Immediate 8bit @3:		Error code for negative battery voltage
;	Immediate 8bit @4:		Error code for low battery voltage
;	out .dseg label @5:		address of calculater battery voltage
;	.cseg label @6:			jump here if error code != ERROR_OK
;	Immediate @7:			V_Drop
;
.MACRO CHK_BT_VOLTAGE
	movw r16,@0 ; calculate one battery voltage
	ldi r18,@7
	add r16,r18
	brcc chk_bt_voltage_02
	inc r17
	chk_bt_voltage_02:
	sub r16,@1
	sbc r17,@2
	STS_WORD @5,r16,r17
	brpl chk_bt_voltage_10 ; Positive?
	ldi ERROR_REG,@4 ; Negative result
	DEBUG_DIAG
	rjmp @6
	
	chk_bt_voltage_10:
	CPI16 r16,r17,V_BATTERY_MIN
	brsh chk_bt_voltage_20
	ldi ERROR_REG,@3
	DEBUG_DIAG
	rjmp @6 ; Error: too low
	chk_bt_voltage_20: ; in allowed range
.ENDMACRO

;-------------------------------------------
;TOGGLE_DIAG_FLAG: change status of a flag
; Usage TOGGLE_DIAG_FLAG @0
;	@0 - Immediate: value of the flag bit. (DIAG_FET_S)
;
.MACRO TOGGLE_DIAG_FLAG 
	lds r18,sram_diag_flags
	ldi r19,@0
	eor r18,r19
	sts sram_diag_flags,r18
.ENDMACRO

;-------------------------------------------
;SET_DIAG_FLAG: set a flag
; Usage SET_DIAG_FLAG @0
;	@0 - Immediate: value of the flag bit. (DIAG_FET_S)
;
.MACRO SET_DIAG_FLAG 
	lds r18,sram_diag_flags
	sbr r18,@0
	sts sram_diag_flags,r18
.ENDMACRO

;-------------------------------------------
;CLR_DIAG_FLAG: clear a flag
; Usage CLR_DIAG_FLAG @0
;	@0 - Immediate: value of the flag bit. (DIAG_FET_S)
;
.MACRO CLR_DIAG_FLAG
	lds r18,sram_diag_flags
	cbr r18,@0
	sts sram_diag_flags,r18
.ENDMACRO


.MACRO BUZZER_ON ;Switch on buzzer
	cbi PORTD,BUZZER_SW
.ENDMACRO

.MACRO BUZZER_OFF ;Switch off buzzer
	sbi PORTD,BUZZER_SW
.ENDMACRO

.MACRO FET_ON
	cbi PORTD,GATE_SW ; Switch on FET
.ENDMACRO

.MACRO FET_OFF
	sbi PORTD,GATE_SW ; Switch off FET
.ENDMACRO

.MACRO POWER_ON
	cbi PORTB,PWR_SW ; Switch on 79L05 by uC
.ENDMACRO

.MACRO POWER_OFF
	sbi PORTB,PWR_SW ; Switch off 79L05 by uC
.ENDMACRO

.MACRO INH_ON
	cbi PORTB,CMUX_INH ;on INH
.ENDMACRO

.MACRO INH_OFF
	sbi PORTB,CMUX_INH ;off INH
.ENDMACRO

; ************** START PROGRAM ******************
.org 0x000
rjmp start

.org INT_VECTORS_SIZE
start:

; Init Stack
ldi r16,HIGH(RAMEND)
out SPH,r16
ldi r16,LOW(RAMEND)
out SPL,r16

;clear sram
ldi r16,0xaa
ldi r30,low(SRAM_START)
ldi r31,high(SRAM_START)
init_10:
st z+,r16
com r16
CPI16 r30,r31,RAMEND+1
brne init_10

clr r16; clear diagnostic flags (usart echo off)
sts sram_diag_flags,r16
.if defined(debugmode) && defined(debugusart)
 .if debugusart == 1; use virtual screen
	ldi r16,low(sram_debug_usart_out)
	ldi r17,high(sram_debug_usart_out)
	sts sram_debug_usart_pnt,r16
	sts sram_debug_usart_pnt+1,r17
 .endif
.endif

; I/O settings
; Pull up resistors enable
in r16, SFIOR
cbr r16, 1<<PUD
out SFIOR,r16

; Set out bits, high states, and Pull-ups on

; PORT B
ldi r16,0xff
ldi r17,(1<<DDB5) | (1<<DDB4) | (1<<DDB3) | (1<<DDB2) | (1<<DDB0)
out PORTB,r16
out DDRB,r17

;PORT D
ldi r16,0xff
ldi r17,(1<<DDD6) | (1<<DDD4) | (1<<DDD2)
out PORTD,r16
out DDRD,r17

POWER_ON; Switch on 79L05 by uC
FET_OFF
BUZZER_OFF
INH_OFF

; Set ADC
ldi r16,(1<<ADEN) | (1<<ADPS2) ; ADC on, CLK/16 (~62KHz), IRQ off, Single mode
out ADCSRA,r16
ldi r16,(1<<REFS0) ; Voltage reference = AVcc, Right Adjust Result
out ADMUX,r16

rcall usart_init ; USART Enable 2400, 8N2

.if defined(debugmode)
	;rcall adc_test; for short test
.endif

rcall diagmode_chk ;check diag mode
breq loop_init
rjmp adc_test ;

; Main loop
loop_init:
clr ERROR_REG
sts sram_drill_stop_cnt,ERROR_REG
ldi ERROR_REG,ERROR_OK
loop:

;uc_levels
ROUND_INIT
loop_uc_levels:
rcall uc_levels
ROUND_CHK 5,loop_uc_levels,close

;heatsink
ROUND_INIT
loop_heatsink:
rcall heatsink
ROUND_CHK 5,loop_heatsink,close

;batt_temp:
ROUND_INIT
loop_batt_temp:
rcall batt_temp
ROUND_CHK 5,loop_batt_temp,close

;batt_voltages
ROUND_INIT
loop_batt_voltages:
rcall batt_voltages
ROUND_CHK 5,loop_batt_voltages,close

;Switch Fet on
FET_ON ; Switch on FET

SET_CMUX CM_DRAIN_D;  check drill working
DEBUG_ADC_RESULT 0x100,0x102
ADC_READ_ALL r16,sram_drain_on,sram_drain_on_ac,sram_drain_on_pp
CPI16 r18,r19,V_DRILL_NOISE; if ac component exist then drill works
brsh loop_working
lds ROUND_REG,sram_drill_stop_cnt
inc ROUND_REG
sts sram_drill_stop_cnt,ROUND_REG
cpi ROUND_REG,20 
brlo loop_end 
rjmp close; if drill stops too long time then go to standby
loop_working:; if drill works, reset the counter
clr ROUND_REG
sts sram_drill_stop_cnt,ROUND_REG

loop_end:
rjmp loop

close: ; go to sleep
sts sram_status,ERROR_REG ; save status to sram

FET_OFF; Switch off FET

cbi ADCSRA,ADIE ; Disable ADC Interrupt
cbi ADCSRA,ADEN ; Disable ADC
in r16,PORTB ; disable 4051 mux and select an empty channel 
ori r16,1<<CMUX_INH
out PORTB,r16
ori r16,CM_EMPTY2
out PORTB,r16

rcall usart_report
lds ERROR_REG,sram_status
rcall buzzer

lds ERROR_REG,sram_status
cpi ERROR_REG,ERROR_OK
brne almost_end
LDS_WORD r16,r17,sram_bt_0
CPI16 r16,r17,1023-V_BATTERIES_NOMINAL
brlo almost_end
ldi ERROR_REG,1; one long beep if battery level is under nominal voltage (U<3*3.6V)
CPI16 r16,r17,1023-V_BATTERIES_LOW
brlo battery_level
ldi ERROR_REG,3; two long beeps if battery level is low (U<3*3V)
battery_level:
DELAY_MILLISEC 1000
rcall buzzer
almost_end:
BUZZER_OFF; Switch off Buzzer
POWER_OFF; Switch off 79L05

endprg:
 rjmp endprg; amg meg nem haltak.

; Hardware specified subroutines

;----------------------------------------------------------------
; uc_levels: read and check uC own voltages. Avcc, Gnd, Bandgap.
; In: - 
; Out:
;	r3:r2, sram_gnd:	GND (uc own gnd)
;   r5:r4, sram_avcc:	Avcc
;	r17:r16, sram_vbg:	Vbandgap
;	ERROR_REG:			Error code
;
uc_levels:
;Read Avcc (ADC1, INH Off)
SET_CMUX CM_EMPTY2
sbi PORTB,CMUX_INH ;disable CMUX
SET_AMUX AM_CMUX
DEBUG_ADC_RESULT_DC 1023 ; for simulator
;DEBUG_ADC_USART sram_sample_avcc
ADC_READ r4,sram_avcc

;Read GND (ADMUX inside uC)
SET_AMUX AM_GND
DEBUG_ADC_RESULT_DC 0 ; for simulator
;DEBUG_ADC_USART sram_sample_gnd
ADC_READ r2,sram_gnd

;Read Vbandgap ~1,3V (ADMUX inside uC)
SET_AMUX AM_VBG
DEBUG_ADC_RESULT_DC VBG_MAX-1; ...-1 is good
;DEBUG_ADC_USART sram_sample_vbg
rcall read_adc
STS_WORD sram_vbg,r16,r17

ldi ERROR_REG,ERROR_OK
CPI16 r16,r17,VBG_MAX ;compare bandgap voltage and Vcc
BRLO uc_levels_end ; Vcc is OK
ldi ERROR_REG, ERROR_BANDGAP; ERROR Vcc is too low
uc_levels_end:
ret

;------------------------------------------------------------------------------------------
;heatsink: read and check heatsink temperature
;In: -
;Out:
;	r17:r16, sram_heatsink:	heatsink temperature
;	ERROR_REG:				error code
;
heatsink:
SET_AMUX AM_HEATSINK
DEBUG_ADC_RESULT_DC V_HEATSINK_MAX-1; -1 is good
rcall read_adc
STS_WORD sram_heatsink,r16,r17
ldi ERROR_REG,ERROR_OK
CPI16 r16,r17,V_HEATSINK_MIN
brsh heatsink10
ldi ERROR_REG,ERROR_HEATSINK_LOW ;ERROR Too low (cold or short circuit)
;DEBUG_DIAG
rjmp heatsink_end

heatsink10:
CPI16 r16,r17,V_HEATSINK_MAX
BRLO heatsink_end
ldi ERROR_REG,ERROR_HEATSINK_HIGH ;ERROR Too high (hot or sensor not connected)
;DEBUG_DIAG
heatsink_end:
ret

;---------------------------------------------------------------------------
;batt_temp: read and check battery temperatures
;In: -
;Out:
;	r7:r6, sram_bt1:	value between battery ptc 1 - battery ptc 2
;	r9:r8, sram_bt2:	value between battery ptc 2 - battery ptc 3
;	r11:r10, sram_bt3:	value between battery ptc 3 - reference resistor
;	sram_btref:			calculated reference resistor value (ADC4-ADC_GND)
;	r15:r14, sram_bt_ratio_min:	calculated low temerature limit
;	r13:r12, sram_bt_ratio_max:	calculated high temperature limit
;	sram_bt1_calc:	calculated temperature of battery 1 (ADC_Avcc-ADC2)
;	sram_bt2_calc:	calculated temperature of battery 2 (ADC2-ADC3)
;	sram_bt3_calc:	calculated temperature of battery 3 (ADC3-ADC4)
;	ERROR_REG:		error code
;	if ERROR_REG != ERROR_OK all calculated results may not stored
;
batt_temp:
;Read battery PTCs
SET_AMUX AM_BT1
DEBUG_ADC_RESULT_DC 1023-230 ;1023-230
;DEBUG_ADC_USART sram_sample_common
ADC_READ r6,sram_bt1

SET_AMUX AM_BT2
DEBUG_ADC_RESULT_DC 333+230; 333+230
;DEBUG_ADC_USART sram_sample_common
ADC_READ r8,sram_bt2

SET_AMUX AM_BT3
DEBUG_ADC_RESULT_DC 333; 333
;DEBUG_ADC_USART sram_sample_common
ADC_READ r10,sram_bt3

movw r12,r10
sub r12,r2 ; r13:r12 = BT reference (BT3-GND)
sbc r13,r3
STS_WORD sram_btref,r12,r13
movw r16,r12
clr r18
ldi  r19,BT_RATIO_MIN
rcall mul16
movw r14,r22 ;BT_MIN value
STS_WORD sram_bt_ratio_min,r14,r15

clr r18
ldi  r19,BT_RATIO_MAX
rcall mul16
movw r12,r22 ;BT_MAX value
STS_WORD sram_bt_ratio_max,r12,r13

ldi ERROR_REG,ERROR_OK
CHK_BT_HEAT r8,r10,r11,ERROR_BT3_HEAT_LOW,ERROR_BT3_HEAT_HIGH,sram_bt3_calc,batt_temp_end ;BT3
CHK_BT_HEAT r6,r8,r9,ERROR_BT2_HEAT_LOW,ERROR_BT2_HEAT_HIGH,sram_bt2_calc,batt_temp_end  ;BT2
CHK_BT_HEAT r4,r6,r7,ERROR_BT1_HEAT_LOW,ERROR_BT1_HEAT_HIGH,sram_bt1_calc,batt_temp_end  ;BT1

batt_temp_end:
ret

;-------------------------------------------------------------------
;batt_voltages: read and check battery voltages
; In: -
; Out:
;	r7:r6,sram_bt_h:	upper battery positive pin (Vcc)
;	r9:r8,sram_bt_m:	between middle battery positive pin and upper battery negative pin
;	r11:r10,sram_bt_l:	between lower battery positive pin and middle battery negative pin
;	r13:r12,sram_bt_0:	lower battery negative pin (0V)
;	sram_bt_h_calc:	calculated upper battery voltage
;	sram_bt_m_calc:	calculated middle battery voltage
;	sram_bt_l_calc:	calculated lower battery voltage
;	ERROR_REG:		error_code
;	if ERROR_REG != ERROR_OK all calculated results may not stored
;
batt_voltages:
SET_AMUX AM_CMUX

SET_CMUX CM_BT_H
DEBUG_ADC_RESULT_DC 1023
;DEBUG_ADC_USART sram_sample_common
ADC_READ r6,sram_bt_h

SET_CMUX CM_BT_M
DEBUG_ADC_RESULT_DC 1023-(1*(V_BATTERY_MIN)) 
;DEBUG_ADC_USART sram_sample_common
ADC_READ r8,sram_bt_m

SET_CMUX CM_BT_L
DEBUG_ADC_RESULT_DC 1023-(2*(V_BATTERY_MIN))
;DEBUG_ADC_USART sram_sample_common
ADC_READ r10,sram_bt_l

SET_CMUX CM_BT_0
DEBUG_ADC_RESULT_DC 1023-(3*(V_BATTERY_MIN))
;DEBUG_ADC_USART sram_sample_common
ADC_READ r12,sram_bt_0

ldi ERROR_REG,ERROR_OK
CHK_BT_VOLTAGE r6,r8,r9,ERROR_BTU_VOLTAGE_LOW,ERROR_BTU_VOLTAGE_NEGATIVE,sram_bt_h_calc,batt_voltages_end,V_DROP; BT1 upper
CHK_BT_VOLTAGE r8,r10,r11,ERROR_BTM_VOLTAGE_LOW,ERROR_BTM_VOLTAGE_NEGATIVE,sram_bt_m_calc,batt_voltages_end,0; BT2 middle
CHK_BT_VOLTAGE r10,r12,r13,ERROR_BTL_VOLTAGE_LOW,ERROR_BTL_VOLTAGE_NEGATIVE,sram_bt_l_calc,batt_voltages_end,0 ; BT3 lower

batt_voltages_end:
ret

;-------------------------------------------------------------------
;buzzer: beeping error code on buzzer
; In: 
;	ERROR_REG: error code 
; Usage: 
;	if ERROR_REG==ERROR_OK: beepeng 3 short signal
;	otherwise beepeing error code in binary from the first "1" bit. Beepeng order: MSB to LSB
;	"1" = long beep
;	"0" = short beep
;
buzzer:
cpi ERROR_REG,ERROR_OK
brne buzzer_error

ldi r20,3 ;Buzzer OK signal: 3 short beep
buzzer_ok_10:
BUZZER_ON
DELAY_MILLISEC 300
BUZZER_OFF
rcall delay 
dec r20
brne buzzer_ok_10
rjmp buzzer_end

buzzer_error:
;BUZZER_ON
;DELAY_MILLISEC 1000
;BUZZER_OFF
;DELAY_MILLISEC 500

ldi r20,8
clr r21
buzzer_error_10:
clr r22
lsl ERROR_REG ; copy next bit to Carry
brcc buzzer_error_20
inc r21 ; found an '1' bit
inc r22 ; store actual bit

buzzer_error_20:
or r21,r21
breq buzzer_error_100 ; ignore leading zeros
BUZZER_ON
lsr r22; read actual bit

brcc buzzer_error_30
DELAY_MILLISEC 400 ; long beep = "1"
buzzer_error_30:   ; short beep = "0"
DELAY_MILLISEC 400 
BUZZER_OFF
DELAY_MILLISEC 400

buzzer_error_100:
dec r20
brne buzzer_error_10
buzzer_end:
BUZZER_OFF
ret

;------------------------------------------------
;diagmode_chk: checking diagnostic mode
; In: 
;	DIAG_MODE_OUT and DIAG_MODE_IN shorted or open
;	shorted:	diagmode on (or debugdiag==1 for simulator)
;	open:		diagmode off
; Out:
;	r16:		0: diagmode off
;				1: diagmode on
;	Z flag:		clear:	diagmode on
;				set:	diagmode off
;
diagmode_chk:
sbi DDRD,DIAG_MODE_OUT ; set dd to output
ldi r16,0
cbi PORTD,DIAG_MODE_OUT
nop
nop
sbis PIND,DIAG_MODE_IN
inc r16
sbi PORTD,DIAG_MODE_OUT
nop
nop
sbic PIND,DIAG_MODE_IN
inc r16
cbi PORTD,DIAG_MODE_OUT
nop
nop
sbis PIND,DIAG_MODE_IN
inc r16
sbi PORTD,DIAG_MODE_OUT
nop
nop
sbic PIND,DIAG_MODE_IN
inc r16
.if defined(debugmode) && defined(debugdiag)
	.if debugdiag == 1 ; 1: forced diag mode for simulator. Don't use on real device!
		ldi r16,4
	.endif
.endif
cpi r16,4
ldi r16,0
brne diag_chk10
inc r16
diag_chk10:
sts sram_diagmode,r16 ; 0 - cont. mode off, 1 - cont. diag mode on 
cbi DDRD,DIAG_MODE_OUT ; set dd to input
or r16,r16 ; Z=1 normal mode, Z=0 diag mode
ret

;------------------------------------------
; Select cd4051 channel (cmux) and enable it
; Usage: rcall sel_cmux
; in
;	r16: pChannel = CM_BT_H, CM_BT_M, ...
; out: -
;
sel_cmux:
push r17
in r17,PORTB
ori r17,1<<CMUX_INH ; disable cmux
out PORTB,r17
andi r17,CM_MASK
or r17,r16 ; set new channel
out PORTB,r17
andi r17,~(1<<CMUX_INH) ; enable cmux
out PORTB,r17
pop r17
ret

;------------------------------------------
;Select ADMUX channel
;Usage rcall sel_amux
;in:
;	r16: pChannel = AM_BT1, AM_BT2, ...
;out: -
;
sel_amux:
push r17
in r17,ADMUX
andi r17,AM_MASK
or r17,r16
out ADMUX,r17
pop r17
ret

;-----------------------------------------------
; read_adc: read 64 sample from selected adc channel
; in: -
; out:
;	r17:r16 - DC component of samples, 10bit resolution
;	r19:r18 - AC component of samples
;	r21:r20 - Maximum value of samples
;	r23:r22 - Minimum value of samples
;	sram_sample_common: sample results
;
read_adc:
.def ADC_DCL = r16 ; DC result Low (also Summ high) FRL
.def ADC_DCH = r17 ; DC result high (also Summ low) FRH
.def ADC_ACL = r18;	AC result low
.def ADC_ACH = r19;	AC result high
.def ADC_MAXL = r20; highest result
.def ADC_MAXH = r21;
.def ADC_MINL = r22; lowest result
.def ADC_MINH = r23; 

.def ADC_RL = r0;	One result low 
.def ADC_RH = r1;	One result high
.def ADC_CNT = r2;	Sample counter

push ADC_RL
push ADC_RH
push ADC_CNT
push ZL
push ZH

rcall read_adc2
in ADC_RL,ADCL
in ADC_RH,ADCH ; discrard 1st result

clr ADC_DCH; DC = 0
clr ADC_DCL
clr ADC_MINL; MIN = 0xffff
dec ADC_MINL 
mov ADC_MINH,ADC_MINL
clr ADC_MAXL; MAX = 0
clr ADC_MAXH

ldi ZL,64
mov ADC_CNT,ZL

ldi ZL,low(sram_sample_common)
ldi ZH,high(sram_sample_common)

r_adc_1:
rcall read_adc2
in ADC_RL,ADCL
in ADC_RH,ADCH
;DEBUG
.if defined(debugmode) && defined(debugadc); using predefined values
	sbrc ADC_CNT,3
	rjmp r_adc_d10
	lds ADC_RL,sram_adc_debug_1
	lds ADC_RH,sram_adc_debug_1+1
	rjmp r_adc_d20
	r_adc_d10:
	lds ADC_RL,sram_adc_debug_2
	lds ADC_RH,sram_adc_debug_2+1
	r_adc_d20:
.endif
;DEBUG END
st z+,ADC_RL ; store for AC
st z+,ADC_RH

cp16 ADC_RL,ADC_RH,ADC_MINL,ADC_MINH
brsh r_adc_11
mov ADC_MINL,ADC_RL
mov ADC_MINH,ADC_RH
r_adc_11:
cp16 ADC_RL,ADC_RH,ADC_MAXL,ADC_MAXH
brlo r_adc_14
mov ADC_MAXL,ADC_RL
mov ADC_MAXH,ADC_RH
r_adc_14:
add ADC_DCH,ADC_RL ; summ results (DCH is low part here!)
adc ADC_DCL,ADC_RH ; (DCL is high part here!)
dec ADC_CNT
brne r_adc_1
ldi ZL,32
add ADC_DCH,ZL ; add "0.5" for round
brcc r_adc_20
inc ADC_DCL
r_adc_20:
ldi ZL,3
lsl ADC_DCH ; div by 64
rol ADC_DCL
rol ADC_DCH
rol ADC_DCL
rol ADC_DCH
and ADC_DCH,ZL ; mask: 8., 9. bit
; ADC_DCH:ADC_DCL contains averaged 10 bit wide DC result

ldi ZL,64 ; AC component
mov ADC_CNT,ZL
ldi ZL,low(sram_sample_common)
ldi ZH,high(sram_sample_common)
clr ADC_ACL
clr ADC_ACH
r_adc_30:
ld ADC_RL,z+
ld ADC_RH,z+
sub ADC_RL,ADC_DCL ; Ut-Udc
sbc ADC_RH,ADC_DCH
brpl r_adc_40

com ADC_RH ; mul by -1
neg ADC_RL
brne r_adc_40
inc ADC_RH

r_adc_40:
add ADC_ACH,ADC_RL ; ACH = low
adc ADC_ACL,ADC_RH ; ACL = high
dec ADC_CNT
brne r_adc_30

ldi ZL,32 ;round +0.5
add ADC_ACH,ZL
brcc r_adc_50
inc ADC_ACL
r_adc_50:

ldi ZL,3
lsl ADC_ACH ; div by 64
rol ADC_ACL
rol ADC_ACH
rol ADC_ACL
rol ADC_ACH
and ADC_ACH,ZL ; mask: 8., 9. bit

pop ZH
pop ZL
pop ADC_CNT
pop ADC_RH
pop ADC_RL
ret

;------------------------------------------------------
;read_adc2: start adc conversion, and waiting until conversion is completed
; In: -
; Out: -
;
read_adc2:
sbi ADCSRA,ADSC ; Start ADC conversion
nop
r_adc2_1: ; wait ACD 
sbic ADCSRA,ADSC
rjmp r_adc2_1
ret

;------------------------------
; General purpose subroutines -
;------------------------------

;---------------------------------------------------------------
;mul16: unsigned multiplication 16bit * 16bit -> 32bit
; M_R3:M_R2:M_R1:M_R0=M_AH:M_AL * M_BH:M_BL unsigned
; r0, r1: reserved
;
mul16:

.def M_AL = r16 ;Source 1 low
.def M_AH = r17 ;Source 1 high
.def M_BL = r18 ;Source 2 low
.def M_BH = r19 ;Source 2 high
.def M_R0 = r20 ;Destination lowest value
.def M_R1 = r21
.def M_R2 = r22
.def M_R3 = r23 ;Destination highest value 

push r0
push r1
mul M_AL,M_AH
mov M_R0,r0
mov M_R1,r1
mul M_AH,M_BH
mov M_R2,r0
mov M_R3,r1
mul M_AL,M_BH
add M_R1,r0
adc M_R2,r1
brcc mul16_2
inc M_R3
mul16_2:
mul M_AH,M_BL
add M_R1,r0
adc M_R2,r1
brcc mul16_3
inc M_R3
mul16_3:
pop r1
pop r0
ret

;-------------------------------------------------
;delay: waiting for specified time
; In:
;	registers: (DELAY_4):DELAY_3:DELAY_2:DELAY_1 
; Out: -
;
delay:
;DEBUG
.if defined(debugmode) && defined(debugdelay)
	ret ;quit fast
.endif
;DEBUG END
 push DELAY_1
 push DELAY_2
 push DELAY_3
 ;push DELAY_4
 com DELAY_1
 com DELAY_2
 com DELAY_3
 ;com DELAY_4
 delay10:
 inc DELAY_1
 brne delay10
 inc DELAY_2
 brne delay10
 inc DELAY_3
 brne delay10
 ;inc DELAY_4
 ;brne delay10
 ;pop DELAY_4
 pop DELAY_3
 pop DELAY_2
 pop DELAY_1
 ret

; USART routines
;----------------------------------------------------------
;usart_init: 
; Enable Tx and Rx
; Set usart 2400 baud 8N2 output
; In: -
; Out: -
;
usart_init:
.equ BAUD = 2400
.equ UBRR = int(1.0*SYSCLK/(16*BAUD))-1 

ldi r16,low(UBRR)
ldi r17,high(UBRR)
out UBRRH,r17
out UBRRL,r16
ldi r16,(1<<RXEN) | (1<<TXEN)
out UCSRB,r16
ldi r16,(1<<URSEL) | (1<<USBS) | (3<<UCSZ0) ; 8N2
out UCSRC,r16
rcall usart_clr_buff
ret

;-----------------------------------------------------
;usart_clr_buff: clear usart Rx buffer
; In: -
; Out: -
;
usart_clr_buff:
push r16
push r30
push r31
ldi r30,low(sram_usart_in)
ldi r31,high(sram_usart_in)
STS_WORD sram_usart_in_pnt,r30,r31
ldi r16,0
u_clr_b10:
st z+,r16
CPI16 r30,r31,sram_usart_in_end
brlo u_clr_b10
pop r31
pop r30
pop r16
ret

;--------------------------------------------------
;usart_read_byte: receive one character from Rx
; In: -
; Out: 
;	r16:	received character
;	Z flag:	clear = error; set = byte received
;
usart_read_byte:
u_read_b10:
rcall diagmode_chk
breq u_read_b100 ; diagmode end (Z=1)
sbis UCSRA,RXC
rjmp u_read_b10
in r16,UDR
push r17
lds r17,sram_diag_flags
sbrc r17,DIAG_ECHO; Echo on?
rcall usart_send_byte ; yes, send received character back
u_read_b20:
pop r17
clz
u_read_b100: ; Z=0: Error, Z=1: - received byte in r16
ret

;-----------------------------------------------
;usart_fill_buff: put received command from usart. 
; Received command placed to usart input puffer. Command terminator: CR or LF
; In: -
; Out:
;	Z flag: set = diagmode off, clear = diagmode on
;	C flag: set = RS232 error, clear = RS232 ok
usart_fill_buff:
rcall usart_clr_buff
LDS_WORD r30,r31,sram_usart_in_pnt
u_fill_b10:
rcall usart_read_byte
breq u_fill_b_err
cpi r16,'\r' ;CR - end
breq u_fill_b_end
cpi r16,'\n' ;LF - end
breq u_fill_b_end
st z+,r16
CPI16 r30,r31,sram_usart_in_end 
brlo u_fill_b10
clz ;diag mode on
sec ;puffer full
rjmp u_fill_b_err
u_fill_b_end:
STS_WORD sram_usart_in_pnt,r30,r31
clz ; diag mode on
clc ; command in puffer
u_fill_b_err: ; Z=1 diagmode off, C=1 RS232 error, C=0 : RS232 ok
ret

;----------------------------------------------------------------
;usart_send_sram_fn: send 64 word wide value from sram
; In:
;	r27:r26: start address
; Out: -
;
usart_send_sram_fn: ; [r27:r26], send 64 word wide value from sram
push r16
push r17
push r18
ldi r18,64

usart_s_sr10:
push r26
push r27
mov r16,r18
andi r16,7
brne usart_s_sr20
ldi r26,low(txt_crlf)
ldi r27,high(txt_crlf)
rcall usart_send_text

usart_s_sr20:
ldi r26,low(txt_hex)
ldi r27,high(txt_hex)
rcall usart_send_text
pop r27
pop r26
ld r16,x+
ld r17,x+
rcall usart_send_hex_word
push r26
push r27
ldi r26,low(txt_comma)
ldi r27,high(txt_comma)
rcall usart_send_text
pop r27
pop r26
dec r18
brne usart_s_sr10

pop r18
pop r17
pop r16
ret

;--------------------------------------------------------------
;usart_send_hex_word: send word wide value in hex format
; In:
;	r17:r16 - word wide value
;
usart_send_hex_word:
push r17
push r16
mov r16,r17
rcall usart_send_hex_byte
pop r16
rcall usart_send_hex_byte
pop r17
ret

;-------------------------------------------------------------
;usart_send_hex_byte: send byte wide value in hex format
; In:
;	r16: 8bit value
; Out: -
;
usart_send_hex_byte:
push r16
push r17
push r30
push r31
mov r17,r16

swap r16 ; upper 4 bit
andi r16,0x0f
ldi r30,low(txt_hex_digits<<1)
ldi r31,high(txt_hex_digits<<1)
add r30,r16
clr r16
adc r31,r16
LPM r16,Z
rcall usart_send_byte
andi r17,0x0f ; lower 4 bit
ldi r30,low(txt_hex_digits<<1)
ldi r31,high(txt_hex_digits<<1)
add r30,r17
clr r16
adc r31,r16
LPM r16,Z
rcall usart_send_byte

pop r31
pop r30
pop r17
pop r16
ret

;------------------------------------------------------
;usart_send_bin_byte: send byte value in binary format
; In:
;	r16: 8bit value
;
usart_send_bin_byte:
push r16
push r17
push r18
mov r17,r16
ldi r18,8
u_s_b_b10:
ldi r16,'0'
lsl r17
brcc u_s_b_b20
inc r16
u_s_b_b20:
rcall usart_send_byte
dec r18
brne u_s_b_b10
pop r18
pop r17
pop r16
ret

;----------------------------------------------------------------
;usart_send_text: send text
; In:
;	r27:r26 - start address (.cseg) of zero terminated text
;Out: -
;
usart_send_text: ;address in r27:r26 (word address)
push r16
push r30
push r31
mov r30,r26
mov r31,r27
lsl r30 ;word addres to byte address
rol r31 

 ;r31:r30 ->Z
usart_s_t10:
LPM r16,Z+
or r16,r16 ; End of text (0x00)? 
breq usart_s_t20
CPI16 r30,r31,(txt_block_end<<1) ; Out of text block?
brsh usart_s_t20
rcall usart_send_byte
rjmp usart_s_t10
usart_s_t20: ;end
pop r31
pop r30
pop r16
ret

;--------------------------------------------------------------------------------
;usart_send_byte: send one byte via usart
; In:
;	r16: byte
; Out: -
;
usart_send_byte:
usart_s_wait:
sbis UCSRA,UDRE
rjmp usart_s_wait
out UDR,r16
.if defined(debugmode) && defined(debugusart)
 .if debugusart == 1; Put character to sram virtual screen
	
	push YL 
	push YH
	push ZL
	push ZH
	push r16
	
	LDS_WORD ZL,ZH,sram_debug_usart_pnt
	CPI16 ZL,ZH,sram_debug_usart_out_end
	brlo usart_s_d30
	ldi YL,low(sram_debug_usart_out+128) ; Scroll up memory. Use breakpoint here to stop to overwrite virtual screen before you read it
	ldi YH,high(sram_debug_usart_out+128)
	
	ldi ZL,low(sram_debug_usart_out)
	ldi ZH,high(sram_debug_usart_out)
	
	usart_s_d10:
	ld r16,y+
	st z+,r16
	CPI16 YL,YH,sram_debug_usart_out_end
	brlo usart_s_d10
	movw YL,ZL
	ldi r16,' '; fill with space
	usart_s_d20:
	st y+,r16
	CPI16 YL,YH,sram_debug_usart_out_end
	brlo usart_s_d20
	nop
	usart_s_d30:
	pop r16
	st z+,r16
	STS_WORD sram_debug_usart_pnt,ZL,ZH
	
	pop ZH
	pop ZL
	pop YH
	pop YL
	
 .endif
.endif
ret

;--------------------------------------------------------------------
;usart_report: send status, measured and calculated results via usart
; In:
;	r31: status1
;	r30: status2
; Out: -
;
usart_report:
;Status, r31, r30
USART_SEND_TXT txt_intro

USART_SEND_TXT txt_status
USART_SEND_REGISTER r31
USART_SEND_TXT txt_comma

USART_SEND_ERROR ERROR_OK,txt_err_ok
USART_SEND_ERROR ERROR_BTU_VOLTAGE_LOW,txt_err_btu_v_l
USART_SEND_ERROR ERROR_BTM_VOLTAGE_LOW,txt_err_btm_v_l
USART_SEND_ERROR ERROR_BTL_VOLTAGE_LOW,txt_err_btl_v_l
USART_SEND_ERROR ERROR_BANDGAP,txt_err_vbg

USART_SEND_ERROR ERROR_HEATSINK_HIGH,txt_err_hs_h
USART_SEND_ERROR ERROR_BT1_HEAT_HIGH,txt_err_bt1_t_h
USART_SEND_ERROR ERROR_BT2_HEAT_HIGH,txt_err_bt2_t_h
USART_SEND_ERROR ERROR_BT3_HEAT_HIGH,txt_err_bt3_t_h

USART_SEND_ERROR ERROR_HEATSINK_LOW,txt_err_hs_l
USART_SEND_ERROR ERROR_BT1_HEAT_LOW,txt_err_bt1_t_l
USART_SEND_ERROR ERROR_BT2_HEAT_LOW,txt_err_bt2_t_l
USART_SEND_ERROR ERROR_BT3_HEAT_LOW,txt_err_bt3_t_l

USART_SEND_ERROR ERROR_BTU_VOLTAGE_NEGATIVE,txt_err_btu_v_n
USART_SEND_ERROR ERROR_BTM_VOLTAGE_NEGATIVE,txt_err_btm_v_n
USART_SEND_ERROR ERROR_BTL_VOLTAGE_NEGATIVE,txt_err_btl_v_n

USART_SEND_TXT txt_round
USART_SEND_REGISTER r30

USART_SEND_TXT txt_crlf
USART_SEND_TXT txt_avcc
USART_SEND_REGISTERS r4,r5
;USART_SEND_SRAM txt_crlf, sram_sample_avcc

USART_SEND_TXT txt_gnd
USART_SEND_REGISTERS r2,r3
;USART_SEND_SRAM txt_crlf, sram_sample_gnd

USART_SEND_TXT txt_bandgap
USART_SEND_VALUE_E1 sram_vbg,ERROR_BANDGAP,txt_high

USART_SEND_TXT txt_comma
USART_SEND_CHAR '('
USART_SEND_IMMEDIATE VBG_MAX
USART_SEND_CHAR ')'

USART_SEND_TXT txt_heatsink
USART_SEND_VALUE_E2 sram_heatsink,ERROR_HEATSINK_HIGH,txt_high,ERROR_HEATSINK_LOW,txt_low

USART_SEND_CHAR ' '
USART_SEND_CHAR '('
USART_SEND_IMMEDIATE V_HEATSINK_MIN
USART_SEND_CHAR '-'
USART_SEND_IMMEDIATE V_HEATSINK_MAX
USART_SEND_CHAR ')'

USART_SEND_TXT txt_bt_adc
USART_SEND_VALUE_E0 sram_bt1
USART_SEND_TXT txt_comma
USART_SEND_VALUE_E0 sram_bt2
USART_SEND_TXT txt_comma
USART_SEND_VALUE_E0 sram_bt3

USART_SEND_TXT txt_bt_ref
USART_SEND_VALUE_E0 sram_btref
USART_SEND_TXT txt_comma
USART_SEND_VALUE_E0 sram_bt_ratio_min
USART_SEND_CHAR '-'
USART_SEND_VALUE_E0 sram_bt_ratio_max

USART_SEND_TXT txt_bt_calc
USART_SEND_VALUE_E2 sram_bt1_calc,ERROR_BT1_HEAT_HIGH,txt_high,ERROR_BT1_HEAT_LOW,txt_low
USART_SEND_TXT txt_comma
USART_SEND_VALUE_E2 sram_bt2_calc,ERROR_BT2_HEAT_HIGH,txt_high,ERROR_BT2_HEAT_LOW,txt_low
USART_SEND_TXT txt_comma
USART_SEND_VALUE_E2 sram_bt3_calc,ERROR_BT3_HEAT_HIGH,txt_high,ERROR_BT3_HEAT_LOW,txt_low

USART_SEND_TXT txt_btv_adc
USART_SEND_VALUE_E0 sram_bt_h
USART_SEND_TXT txt_comma
USART_SEND_VALUE_E0 sram_bt_m
USART_SEND_TXT txt_comma
USART_SEND_VALUE_E0 sram_bt_l
USART_SEND_TXT txt_comma
USART_SEND_VALUE_E0 sram_bt_0

USART_SEND_TXT txt_btv_calc
USART_SEND_VALUE_E2 sram_bt_h_calc,ERROR_BTU_VOLTAGE_LOW,txt_low,ERROR_BTU_VOLTAGE_NEGATIVE,txt_neg
USART_SEND_TXT txt_comma
USART_SEND_VALUE_E2 sram_bt_m_calc,ERROR_BTM_VOLTAGE_LOW,txt_low,ERROR_BTM_VOLTAGE_NEGATIVE,txt_neg
USART_SEND_TXT txt_comma
USART_SEND_VALUE_E2 sram_bt_l_calc,ERROR_BTL_VOLTAGE_LOW,txt_low,ERROR_BTL_VOLTAGE_NEGATIVE,txt_neg
USART_SEND_CHAR ' '
USART_SEND_CHAR '('
USART_SEND_IMMEDIATE V_BATTERY_MIN
USART_SEND_CHAR ')'

USART_SEND_TXT txt_drain_on
USART_SEND_VALUE_E0 sram_drain_on
USART_SEND_TXT txt_comma
USART_SEND_VALUE_E0 sram_drain_on_ac
USART_SEND_TXT txt_comma
USART_SEND_VALUE_E0 sram_drain_on_pp

USART_SEND_TXT txt_adc5
USART_SEND_VALUE_E0 sram_adc5

USART_SEND_TXT txt_crlf2

ret


;-------------------------------------------------------------
;usart_test: testing usart communication
; In: -
; Out: -
;
usart_test:
	ldi r16,65
	rcall usart_send_byte
	ldi r16,66
	rcall usart_send_byte
	ldi r16,67
	rcall usart_send_byte
	ldi r16,13
	rcall usart_send_byte
	ldi r16,10
	rcall usart_send_byte
	ldi r18,1
	ldi r17,0
usart_test_lufi:
	mov r16,r18
	rcall usart_send_hex_word
	ldi r16,13
	rcall usart_send_byte
	ldi r16,10
	rcall usart_send_byte
	lsl r18
	rol r17
	brcc usart_test_lufi
	
	ldi r26,low(txt_samples)
	ldi r27,high(txt_samples)
	rcall usart_send_text
	ldi r26,low(txt_crlf)
	ldi r27,high(txt_crlf)
	rcall usart_send_text

	in r16,SPL
	in r17,SPH
	rcall usart_send_hex_word
	ldi r26,low(txt_crlf)
	ldi r27,high(txt_crlf)
	rcall usart_send_text

	ldi r26,low(SRAM_START)
	ldi r27,high(SRAM_START)
	rcall usart_send_sram_fn

	ldi r26,low(txt_crlf)
	ldi r27,high(txt_crlf)
	rcall usart_send_text

	in r16,SPL
	in r17,SPH
	rcall usart_send_hex_word
	ldi r26,low(txt_crlf)
	ldi r27,high(txt_crlf)
	rcall usart_send_text
	
;usart_test_lufivege:
;	rjmp usart_test_lufivege
ret

;----------------------------------------------------------------------
;adc_test: control and monitor drill controller via terminal emulation
; In: -
; Out: -
;
adc_test:
clr r16 ; reset diagnostic mode basic variables
sts sram_diag_amux,r16
sts sram_diag_cmux,r16
sts sram_diag_flags,r16
SET_DIAG_FLAG DIAG_FIRST_RUN_S
rcall adc_t_help 
rcall adc_t_set
adc_t_loop:
USART_SEND_TXT txt_ready
rcall usart_fill_buff ; read command from terminal
brne adc_t_10; diagmode on
rjmp adc_t_end; diagmode off
adc_t_10:
brcc adc_t_20; Receiving ok?
USART_SEND_TXT txt_rserror
rjmp adc_t_loop
adc_t_20:
ldi r30,low(sram_usart_in)
ldi r31,high(sram_usart_in)
ld r16,z+
ld r17,z+

cpi r16,'h' ; HELP
brne adc_t_30
rcall adc_t_help
rcall adc_t_head
rjmp adc_t_loop
adc_t_30:

cpi r16,'q' ; QUIT
brne adc_t_32
rjmp adc_t_end
adc_t_32:

cpi r16,'i' ; toggle INH
brne adc_t_40
TOGGLE_DIAG_FLAG DIAG_INH_S
rcall adc_t_set
rjmp adc_t_loop
adc_t_40:

cpi r16,'f' ; toggle FET
brne adc_t_44
TOGGLE_DIAG_FLAG DIAG_FET_S
rcall adc_t_set
rjmp adc_t_loop
adc_t_44:

cpi r16,'b' ; toggle BUZZER
brne adc_t_46
TOGGLE_DIAG_FLAG DIAG_BUZZER_S
rcall adc_t_set
rjmp adc_t_loop
adc_t_46:

cpi r16,'e' ; toggle ECHO
brne adc_t_48
TOGGLE_DIAG_FLAG DIAG_ECHO_S
rcall adc_t_set
rjmp adc_t_loop
adc_t_48:

cpi r16,'m' ; toggle measure mode
brne adc_t_50
TOGGLE_DIAG_FLAG DIAG_MEASURE_MODE_S
rcall adc_t_set
rjmp adc_t_loop
adc_t_50:

cpi r16,'s'; start measures
brne adc_t_52
rcall adc_t_meas
rjmp adc_t_loop
adc_t_52:

cpi r17,0 ; EOL ?
breq adc_t_90

cpi r17,'a' ; convert hex character to number
brlo adc_t_70
subi r17,'a'-10-'0'
adc_t_70:
subi r17,'0'
andi r17,15


cpi r16,'a' ; AMUX
brne adc_t_80
sts sram_diag_amux,r17
rcall adc_t_set
rjmp adc_t_loop
adc_t_80:

cpi r16,'c' ; CMUX
brne adc_t_90
andi r17,7
sts sram_diag_cmux,r17
rcall adc_t_set
rjmp adc_t_loop

adc_t_90: ; unknown or wrong command
rcall adc_t_head
rjmp adc_t_loop

adc_t_end:
clr r16 ; reset diagnostic mode basic variables
sts sram_diag_amux,r16
sts sram_diag_cmux,r16
sts sram_diag_flags,r16
rcall adc_t_set
rjmp almost_end
ret

;--------------------------------------------------------------------
;adc_t_meas: measure short series in different time
; In: -
; Out: -
;
adc_t_meas:
lds r16,sram_diag_flags
sbrs r16,DIAG_MEASURE_MODE
rjmp adc_t_m20
ldi r30,low(sram_sample_common) ;short series mode
ldi r31,high(sram_sample_common)
rcall adc_t_meas2; fastest result after settings
DELAY_MILLISEC 10
rcall adc_t_meas2; 10 ms later
DELAY_MILLISEC 40
rcall adc_t_meas2; 50 ms later
DELAY_MILLISEC 50
rcall adc_t_meas2; 100 ms later
DELAY_MILLISEC 100
rcall adc_t_meas2; 200 ms later
DELAY_MILLISEC 300
rcall adc_t_meas2; 500 ms later
DELAY_MILLISEC 500
rcall adc_t_meas2; 1000 ms later
DELAY_MILLISEC 500
rcall adc_t_meas2; 1500 ms later

rjmp adc_t_m_end

adc_t_m20: ;normal mode
DEBUG_ADC_RESULT 100,300
rcall read_adc
STS_WORD sram_diag_adc_dc,r16,r17
STS_WORD sram_diag_adc_ac,r18,r19
STS_WORD sram_diag_adc_max,r20,r21
STS_WORD sram_diag_adc_min,r22,r23
sub r20,r22
sbc r21,r23
STS_WORD sram_diag_adc_pp,r20,r21

adc_t_m_end:
rcall adc_t_send_res
ret

;--------------------------------------------------------------
;adc_t_meas2: read 8 result from adc
; In: -
; Out: -
;
adc_t_meas2:
ldi r18,8
adc_t_m2_10:
rcall read_adc2
in r16,ADCL
in r17,ADCH
st z+,r16
st z+,r17
dec r18
brne adc_t_m2_10
ret

;---------------------------------------------------------------
;adc_t_send_res: send measured series to termninal
; In: -
; Out: -
;
adc_t_send_res:
lds r16,sram_diag_flags
andi r16,DIAG_MEASURE_MODE_S
breq adc_t_s_r10
USART_SEND_TXT txt_adc_results_short
rjmp adc_t_s_r50
adc_t_s_r10:
USART_SEND_TXT txt_adc_results_normal

LDS_WORD r16,r17,sram_diag_adc_dc
USART_SEND_REGISTERS r16,r17
USART_SEND_TXT txt_comma

LDS_WORD r16,r17,sram_diag_adc_ac
USART_SEND_REGISTERS r16,r17
USART_SEND_TXT txt_comma

LDS_WORD r16,r17,sram_diag_adc_pp
USART_SEND_REGISTERS r16,r17
USART_SEND_TXT txt_comma

LDS_WORD r16,r17,sram_diag_adc_max
USART_SEND_REGISTERS r16,r17
USART_SEND_TXT txt_comma

LDS_WORD r16,r17,sram_diag_adc_min
USART_SEND_REGISTERS r16,r17

USART_SEND_TXT txt_crlf
adc_t_s_r50:
ldi r30,low(sram_sample_common)
ldi r31,high(sram_sample_common)

ldi r16,8
adc_t_s_r52:
push r16
rcall adc_t_send_res2
USART_SEND_TXT txt_crlf
pop r16
dec r16
brne adc_t_s_r52
ret

;----------------------------------------------------------------
;adc_t_send_res2:  send 8 adc result to terminal
; In: -
; Out: -
;
adc_t_send_res2: ; send next 8 results
ldi r18,8
adc_t_sr2_10:
ld r16,z+
ld r17,z+
USART_SEND_REGISTERS r16,r17
USART_SEND_TXT txt_comma
dec r18
brne adc_t_sr2_10
ret

;----------------------------------------------------
;adc_t_help: send adc_test help page to terminal
; In: -
; Out: - 
;
adc_t_help:
USART_SEND_TXT txt_adc_help1
USART_SEND_TXT txt_adc_help2
USART_SEND_TXT txt_adc_help3
USART_SEND_TXT txt_adc_help4
USART_SEND_TXT txt_adc_help5
USART_SEND_TXT txt_adc_help6
USART_SEND_TXT txt_adc_help7
USART_SEND_TXT txt_adc_help8
USART_SEND_TXT txt_adc_help9
USART_SEND_TXT txt_adc_help10
USART_SEND_TXT txt_adc_help11
USART_SEND_TXT txt_crlf2
ret

;----------------------------------------------------
;adc_t_set: set amux, cmux, fet, inh. Start measuring, send results to terminal
; In: -
; Out: -
;
adc_t_set: ; SET the setting
lds r16,sram_diag_amux ;amux
rcall sel_amux

lds r17,sram_diag_cmux ;CMUX
ldi r16,0; flip bit order
lsr r17
rol r16
lsr r17
rol r16
lsr r17
rol r16

lsl r16 ;shift to PB3-PB5 
lsl r16
lsl r16
rcall sel_cmux

lds r16,sram_diag_flags; Set INH
sbrs r16,DIAG_INH
rjmp adc_t_set_10
INH_ON
rjmp adc_t_set_12
adc_t_set_10:
INH_OFF
adc_t_set_12:

sbrs r16,DIAG_FET; Set FET
rjmp adc_t_set_16
FET_ON
rjmp adc_t_set_18
adc_t_set_16:
FET_OFF
adc_t_set_18:

sbrs r16,DIAG_BUZZER; Set Buzzer
rjmp adc_t_set_22
BUZZER_ON
rjmp adc_t_set_24
adc_t_set_22:
BUZZER_OFF
adc_t_set_24:

adc_t_am_end:
lds r16,sram_diag_flags; skip measuring if run first time
sbrs r16,DIAG_FIRST_RUN
rcall adc_t_meas
CLR_DIAG_FLAG DIAG_FIRST_RUN_S
rcall adc_t_head
ret

;------------------------------------------------------------------------
;adc_t_head: print adc_test status to terminal
; In: -
; Out: -
;
adc_t_head:

USART_SEND_TXT txt_crlf
USART_SEND_TXT txt_amux
lds r16,sram_diag_amux
USART_SEND_REGISTER r16

USART_SEND_TXT txt_cmux
lds r16,sram_diag_cmux
USART_SEND_REGISTER r16

USART_SEND_TXT txt_inh
lds r16,sram_diag_flags
andi r16,DIAG_INH_S
breq adc_t_head10
USART_SEND_TXT txt_on
rjmp adc_t_head12
adc_t_head10:
USART_SEND_TXT txt_off
adc_t_head12:

USART_SEND_TXT txt_fet
lds r16,sram_diag_flags
andi r16,DIAG_FET_S
breq adc_t_head16
USART_SEND_TXT txt_on
rjmp adc_t_head18
adc_t_head16:
USART_SEND_TXT txt_off
adc_t_head18:

USART_SEND_TXT txt_buzzer
lds r16,sram_diag_flags
andi r16,DIAG_BUZZER_S
breq adc_t_head22
USART_SEND_TXT txt_on
rjmp adc_t_head24
adc_t_head22:
USART_SEND_TXT txt_off
adc_t_head24:

USART_SEND_TXT txt_echo
lds r16,sram_diag_flags
andi r16,DIAG_ECHO_S
breq adc_t_head28
USART_SEND_TXT txt_on
rjmp adc_t_head30
adc_t_head28:
USART_SEND_TXT txt_off
adc_t_head30:

USART_SEND_TXT txt_sample_m
lds r16,sram_diag_flags
andi r16,DIAG_MEASURE_MODE_S
breq adc_t_head34
USART_SEND_TXT txt_short_series
rjmp adc_t_head36
adc_t_head34:
USART_SEND_TXT txt_normal
adc_t_head36:

USART_SEND_TXT txt_admux
in r16,ADMUX
USART_SEND_BIN_REG r16

USART_SEND_TXT txt_ddr
in r16,DDRB
USART_SEND_BIN_REG r16
USART_SEND_TXT txt_comma
in r16,DDRD
USART_SEND_BIN_REG r16

USART_SEND_TXT txt_port
in r16,PORTB
USART_SEND_BIN_REG r16
USART_SEND_TXT txt_comma
in r16,PORTD
USART_SEND_BIN_REG r16

USART_SEND_TXT txt_pin
in r16,PINB
USART_SEND_BIN_REG r16
USART_SEND_TXT txt_comma
in r16,PIND
USART_SEND_BIN_REG r16
USART_SEND_TXT txt_crlf
ret


;------------------------------------------------------
; serial communication constans
;
txt_crlf3:
.db '\r', '\n'
txt_crlf2:
.db '\r', '\n'
txt_crlf:
.db '\r', '\n', 0

txt_hex:
.db "0x",0

txt_bin:
.db "0b",0

txt_comma:
.db ", ",0

txt_hex_digits:
.db "0123456789abcdef",0

txt_average:
.db "Average: ",0

txt_samples:
.db "Samples: ",0

txt_compare:
.db "Compare: ",0

txt_status:
.db '\r','\n',"Status: ",0

txt_round:
.db '\r','\n',"Round: ",0

txt_on:
.db "On.",0

txt_off:
.db "Off",0

txt_normal:
.db "Normal",0

txt_short_series:
.db "Short series",0

txt_dummy:
.db "Dummy: ",0

txt_high:
.db " High ",0

txt_low:
.db " Low ",0

txt_neg:
.db " Negative ",0

txt_intro:
.db '\r','\n','\r','\n',"Cordless drill controller diagnostic report",'\r','\n','\r','\n',0

txt_adc_help1:
.db '\r','\n',"ADC Diagnostic mode help:",0

txt_adc_help2:
.db '\r','\n',"ADMUX selection: ax+[CR], where x=0..f",0

txt_adc_help3:
.db '\r','\n',"CMUX selection: cx+[CR], where x=0..7",0

txt_adc_help4:
.db '\r','\n',"Toggle CMUX INH: i+[CR]",0

txt_adc_help5:
.db '\r','\n',"Toggle FET: f+[CR]",0

txt_adc_help6:
.db '\r','\n',"Toggle buzzer: b+[CR]",0

txt_adc_help7:
.db '\r','\n',"Toggle measuring mode: m+[CR]",0

txt_adc_help8:
.db '\r','\n',"Start adc conversion: s+[CR]",0

txt_adc_help9:
.db '\r','\n',"Toggle echo: e+[CR]",0

txt_adc_help10:
.db '\r','\n',"Send temperature information to the battery charger: t+[CR]",0

txt_adc_help11:
.db '\r','\n',"Help: h+[CR]",0

txt_amux:
.db "AMUX: ",0

txt_cmux:
.db ", CMUX: ",0

txt_inh:
.db ", INH: ",0

txt_fet:
.db ", FET: ",0

txt_buzzer:
.db ", Buzzer: ",0

txt_echo:
.db ", Echo: ",0

txt_sample_m:
.db ", Sample mode: ",0


txt_admux:
.db '\r','\n',"ADMUX:    ",0

txt_DDR:
.db '\r','\n',"DDR  B,D: ",0

txt_PORT:
.db '\r','\n',"PORT B,D: ",0

txt_PIN:
.db '\r','\n',"PIN  B,D: ",0

txt_rserror:
.db '\r','\n',"RS Error",'\r','\n',0

txt_adc_results_short:
.db '\r','\n',"ADC results. 8 - 8 samples in 0sec, 10msec, 50ms, 100ms, 200ms, 500ms, 1s, 1.5s:",'\r','\n',0

txt_adc_results_normal:
.db '\r','\n',"ADC results. DC, AC, Peak to peak, MAX, MIN, Samples:",'\r','\n',0


txt_gnd:
.db '\r','\n',"Gnd: ",0

txt_avcc:
.db '\r','\n',"AVcc: ",0

txt_bandgap:
.db '\r','\n',"Bandgap voltage, (limit): ",0

txt_heatsink:
.db '\r','\n',"Heatsink, (limits): ",0

txt_bt_adc:
.db '\r','\n',"Battery temperature:",'\r','\n',"ADC results: ",0

txt_bt_ref:
.db '\r','\n',"Reference, calculated limits: ",0

txt_bt_calc:
.db '\r','\n',"Calculated BT1, BT2, BT3: ",0

txt_btv_adc:
.db '\r','\n',"Battery voltages:",'\r','\n',"ADC results (u+, u-m, m-l, 0): ",0

txt_btv_calc:
.db '\r','\n',"Calculated Vu, Vm, Vl, (limit): ",0

txt_drain_t_off:
.db '\r','\n',"Drain transistor (fet off), (limit): ",0

txt_drain_t_on:
.db '\r','\n',"Drain transistor (fet on): ",0

txt_drain_on:
.db '\r','\n',"Drain fet 'on' value (dc, ac, peak to peak): ",0

txt_drain_off:
.db '\r','\n',"Drain fet 'off' value: ",0

txt_adc5:
.db '\r','\n',"ADC5 value: ",0

txt_ready:
.db "Ready. (Help: h+[CR])",'\r','\n',0

txt_err_ok:
.db "Ok.",0

txt_err_btu_v_l:
.db "Upper battery voltage is low.",0

txt_err_btu_v_n:
.db "Upper battery voltage is negative.",0

txt_err_btm_v_l:
.db "Middle battery voltage is low.",0

txt_err_btm_v_n:
.db "Middle battery voltage is negative.",0

txt_err_btl_v_l:
.db "Lower battery voltage is low.",0

txt_err_btl_v_n:
.db "Lower battery voltage is negative.",0

txt_err_vbg:
.db "Bandgap value high -> Vcc is low.",0

txt_err_hs_h:
.db "Heatsink temperature is high.",0

txt_err_hs_l:
.db "Heatsink temperature is low.",0

txt_err_bt1_t_h:
.db "Battery 1 temperature is high.",0

txt_err_bt2_t_h:
.db "Battery 2 temperature is high.",0

txt_err_bt3_t_h:
.db "Battery 3 temperature is high.",0

txt_err_bt1_t_l:
.db "Battery 1 temperature is low.",0

txt_err_bt2_t_l:
.db "Battery 2 temperature is low.",0

txt_err_bt3_t_l:
.db "Battery 3 temperature is low.",0

.db 0
txt_block_end: ; End of text block


;data segment
.dseg
.org SRAM_START

; ADC results
sram_gnd: .byte 2;		ADC inner GND (GND means uC power supply negative pin voltage. (Vbattery positive output - 5V). 0V means negative output of battery pack. 
sram_avcc: .byte 2;	ADC1: Avcc  (4051 output disabled)
sram_vbg: .byte 2;		ADC inner Vbandgap 
sram_heatsink: .byte 2;ADC0 Heatsink ptc
sram_bt1: .byte 2;	ADC2 between battery ptc 1 - battery ptc 2
sram_bt2: .byte 2;	ADC3 between battery ptc 2 - battery ptc 3
sram_bt3: .byte 2;	ADC4 between battery ptc 3 - reference resistor
sram_btref: .byte 2;	calculated reference resistor value (ADC4-ADC_GND)
sram_bt1_calc: .byte 2;	calculated temperature of battery 1 (ADC_Avcc-ADC2)
sram_bt2_calc: .byte 2;	calculated temperature of battery 2 (ADC2-ADC3)
sram_bt3_calc: .byte 2;	calculated temperature of battery 3 (ADC3-ADC4)
sram_bt_ratio_min: .byte 2;	calculated low temerature limit
sram_bt_ratio_max: .byte 2;	calculated high temperature limit
sram_bt_h: .byte 2;	ADC1 - 4051 - Vcc	
sram_bt_m: .byte 2;	ADC1 - 4051 - between upper and middle batteries
sram_bt_l: .byte 2;	ADC1 - 4051 - between middle and lower batteries
sram_bt_0: .byte 2;	ADC1 - 4051 - 0V (0V != GND)
sram_bt_0_ac: .byte 2; AC component
sram_bt_0_pp: .byte 2; peak to peak component
sram_bt_h_calc: .byte 2;	calculated upper battery voltage
sram_bt_m_calc: .byte 2;	calculated middle battery voltage
sram_bt_l_calc: .byte 2;	calculated lower battery voltage
sram_drain_on: .byte 2;	ADC1 - 4051 - collector voltage of drain voltage controlled transistor
sram_drain_on_ac: .byte 2; AC component
sram_drain_on_pp: .byte 2; Peak to peak component
sram_drain_off: .byte 2;	ADC1 - 4051 - collector voltage of drain voltage controlled transistor 
sram_drain_t_on: .byte 2;	ADC1 - 4051 - collector voltage of drain voltage controlled transistor
sram_drain_t_off: .byte 2;	ADC1 - 4051 - collector voltage of drain voltage controlled transistor 
sram_empty1: .byte 2;		ADC1 - 4051 - n.c. pin X5
sram_empty2: .byte 2;		ADC1 - 4051 - n.c. pin X7
sram_adc5: .byte 2;			ADC5 result
sram_status: .byte 1; r31 - Status (Error codes)
sram_round: .byte 1; Error checking counter
sram_drill_stop_cnt: .byte 1; counting the amuont of time when drill stops
sram_diagmode: .byte 1; Diag mode 
sram_diag_amux: .byte 1; amux in diag mode
sram_diag_cmux: .byte 1; cmux in diag mode
sram_diag_flags: .byte 1; FET, INH, Buzzer, Echo state in diag mode
sram_diag_adc_dc: .byte 2
sram_diag_adc_ac: .byte 2
sram_diag_adc_max: .byte 2
sram_diag_adc_min: .byte 2
sram_diag_adc_pp: .byte 2

sram_usart_in_pnt: .byte 2
sram_usart_in: .byte 16
sram_usart_in_end: .byte 0
sram_sample_common: .byte 128
;DEBUG
.if defined(debugmode) && defined(debugadc); using predefined values
	sram_adc_debug_1: .byte 2
	sram_adc_debug_2: .byte 2
.endif
.if defined(debugmode) && defined(debugusart) ; reserving memory for virtual screen
	.if debugusart == 1 // in simulator
		sram_debug_usart_pnt: .byte 2
		sram_debug_usart_out: .byte 512
		sram_debug_usart_out_end: .byte 0 
	.endif
.endif
;DEBUG
